#   Copyright (c) 2011, Canonical Ltd
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

from functools import partial
from optparse import OptionParser
from StringIO import StringIO
import sys

import bson
import gpgme
import oops
from oops_datedir_repo import DateDirRepo
import web

class verify:
    def GET(self):
        return (
                "POST to this URL to verify a signature. "
                "The result will be a bson format dict with three keys: "
                "timestamp, signed-content and signer (a key fingerprint). "
                "Errors result in a 500 HTTP status code with body describing "
                "the fault."
                )

    def POST(self):
        try:
            context = gpgme.Context()
        except gpgme.GpgmeError, e:
            # app.internalerrror oopses: unexpected sitation
            raise app.internalerror(e.message)
        data = web.data()
        signed = StringIO(data)
        plaintext = StringIO()
        try:
            signatures = context.verify(signed, None, plaintext)
        except gpgme.GpgmeError, e:
            # e.g. Not signed at all: not an oops
            raise web.webapi.internalerror(e.message)
        if len(signatures) != 1:
            raise web.webapi.internalerror(
                    "Either no signatures, or too many.")
        if signatures[0].status is not None:
            # e.g. GpgmeError(7, 8, u'Bad signature') if given a detached sig.
            # - not an oops
            raise web.webapi.internalerror(str(signatures[0].status.args))
        result = {
            'timestamp': signatures[0].timestamp,
            'signed-content': plaintext.getvalue(),
            'signer': signatures[0].fpr,
            }
        return bson.dumps(result)

urls = (
  '/', 'verify',
  )

app = web.application(urls, globals(), autoreload=False)

def make_oops_error(config, orig_internalerror):
    def _oops_error():
        exc_info = sys.exc_info()
        report = config.create(dict(exc_info=exc_info, url=web.ctx.path))
        ids = config.publish(report)
        if not ids:
            return orig_internalerror()
        else:
            message="Internal server error OOPS-ID: %s" % report['id']
            return web.webapi.HTTPError(
                "500 %s" % message,
                {'Content-Type': 'text/plain', 'OOPS-ID': str(report['id'])},
                message)
    return _oops_error


if __name__ == "__main__":
    web.config.debug = False
    parser = OptionParser()
    parser.add_option("--oops-root", dest="oops_dir",
            help="The root directory to write OOPS reports to.", default=".")
    parser.add_option("--oops-instance", dest="oops_prefix",
            help=("The instance id for this service, should uniquely "
            "identify the running instance."), default="GPGVERIFY")
    options, args = parser.parse_args()
    oops_config = oops.Config()
    oops_repo = DateDirRepo(options.oops_dir, options.oops_prefix)
    oops_config.publishers.append(oops_repo.publish)
    app.internalerror = make_oops_error(oops_config, app.internalerror)
    app.run()
